unit MPEGPlus;
{


Class TMPEGPlus
(c)2001 Copyright by Jurgen Faul
--------------------------------

This class gives you information about MPEG Plus files (*.mpc/*.mp+).
It allows very easy reading and writing of MPEG data (TAG and HEADER).

Supported format is MPEG Plus stream version 7.
MPEG TAGs are also supported (ID3v1.1).

Compiled and tested with Delphi 5.

If you are interested to read about MPEG Plus take a look at
http://www.mpegplus.org


Copyright notice:
-----------------

All rights reserved by the author.
This program is freeware for all purposes.


Contact info:
-------------

E-mail: jfaul@gmx.de
Spoken languages: English, German, Russian

If you are interested for a new version look at
http://www.jfaul.de/mpegplus

Support is available only through the address written above.
No other contact methods are possible. Remember this is a freeware
project, therefore I cannot dedicate too much time and efforts to it.
I will do my best to respond to any mail regarding this subject.
You are welcome to send comments and suggestions to the author.
You are also welcome to support the author by donation of literature,
tools, software or money, but it is not reqired.


Versions:
---------

1.0 (05/21/2001)
- Support for MPEG Plus stream version 7 only
- Read ID3v1.1 tag


}


interface

uses SysUtils;

const
  // Stream versions
  MPP_STREAM_VERSION = 7;

  // Channel mode
  MPP_CM_STEREO = 1;
  MPP_CM_JOINT_STEREO = 2;

  // Sample rate
  MPP_SAMPLE_RATE = 44100;

type
  String3 = string[3];
  String4 = string[4];
  String30 = string[30];
  String255 = string[255];

  // Data structure of TMPEGPlus
  TMPPData = packed record
    Header: String3;        // Tag header
    Title: String30;        // Song title
    Artist: String30;       // Artist name
    Album: String30;        // Album
    Year: String4;          // Year
    Comment: String30;      // Comment
    Genre: byte;            // Genre code
    Track: byte;            // Track number on album
    Duration: word;         // Song duration
    FileLength: longint;    // File length
    SampleRate: word;       // Sampling rate in Hz
    BitRate: word;          // Bitrate
    Mode: byte;             // 1 - Stereo, 2 - Joint-Stereo
    FileName: String255;    // File name
  end;

  TMPEGPlus = class(TObject)
    private
      FData: TMPPData;
      function FGetFileName: string;
      procedure FSetFileName(InStr: string);
      function FGetIsValid: boolean;
      function fGetIsTagged: boolean;
      function FReadData: integer;
    public
      constructor Create;
      procedure ResetData;
      property IsValid: boolean read FGetIsValid;
      property IsTagged: boolean read fGetIsTagged;
      property Title: String30 read FData.Title;
      property Artist: String30 read FData.Artist;
      property Album: String30 read FData.Album;
      property Year: String4 read FData.Year;
      property Comment: String30 read FData.Comment;
      property Genre: byte read FData.Genre;
      Property Track: byte read FData.Track;
      property Duration: word read FData.Duration;
      property FileLength: longint read FData.FileLength;
      property SampleRate: word read FData.SampleRate;
      property BitRate: word read FData.BitRate;
      property Mode: byte read FData.Mode;
      property FileName: string read FGetFileName write FSetFileName;
  end;


implementation

type
  TTag = packed record
    Header: array[1..3] of char;
    Title: array[1..30] of char;
    Artist: array[1..30] of char;
    Album: array[1..30] of char;
    Year: array[1..4] of char;
    Comment: array[1..30] of char;
    Genre: byte;
  end;

constructor TMPEGPlus.Create;
begin
	inherited Create;
end;

function TMPEGPlus.FGetFileName: string;
begin
  Result := FData.FileName;
end;

procedure TMPEGPlus.FSetFileName(InStr: string);
begin
  FData.FileName := InStr;
  FReadData;
end;

function TMPEGPlus.FGetIsValid: boolean;
begin
  Result := FData.Duration > 0;
end;

function TMPEGPlus.FGetIsTagged: boolean;
begin
  Result := FData.Header = 'TAG';
end;

procedure TMPEGPlus.ResetData;
begin
  with FData do
  begin
    Header := '';
    Title := '';
    Artist := '';
    Album := '';
    Year := '';
    Comment := '';
    Genre := 255;
    Track := 0;
    Duration := 0;
    FileLength := 0;
    SampleRate := 0;
    Mode := 0;
    BitRate := 0;
  end;
end;

function TMPEGPlus.FReadData: integer;
var
  Source: file;
  FourBytesString: array [1..4] of char;
  FourBytesLongint: longint absolute FourBytesString;
  TempByte: byte;
  Tag: TTag;
begin
  ResetData;
  Result := -1;

  If FileExists(FData.FileName) then
  begin
    AssignFile(Source, FData.FileName);

    try
      {$I-}
      Reset(Source, 1);
      {$I+}
      Result := IOResult;

      if (Result = 0) and (FileSize(Source) > 100) then
      begin
        FData.FileLength := FileSize(Source);

        // Check file format
        BlockRead(Source, FourBytesString , 4);
        if (FourBytesString[1] <> 'M') or
          (FourBytesString[2] <> 'P') or
          (FourBytesString[3] <> '+') or
          (FourBytesString[4] < #7) then raise Exception.Create('');

        // Read duration and calculate bitrate
        BlockRead(Source, FourBytesString , 4);
        FData.SampleRate := MPP_SAMPLE_RATE;
        FData.Duration := Round((1152 / FData.SampleRate) * FourBytesLongint);
        if FData.Duration <= 0 then raise Exception.Create('');
        FData.BitRate := Round(FData.FileLength / FData.Duration / 125);
        if (FData.BitRate < 32) or (FData.BitRate > 320) then
          raise Exception.Create('');

        // Read channel mode
        BlockRead(Source, FourBytesString , 4);
        TempByte := Ord(FourBytesString[4]);
        TempByte := TempByte mod 128;
        if TempByte div 64 = 0 then FData.Mode := MPP_CM_STEREO
        else FData.Mode := MPP_CM_JOINT_STEREO;

        // Read ID3-tag
        if FData.FileLength > 128 then
        begin
          Seek(Source, FData.FileLength - 128);
          BlockRead(Source, Tag, 128);
          if Tag.Header='TAG' then
          begin
            FData.Header := Trim(Tag.Header);
            FData.Title := Trim(Tag.Title);
            FData.Artist := TrimRight(Tag.Artist);
            FData.Album := TrimRight(Tag.Album);
            FData.Year := Tag.Year;
            FData.Comment := Trim(Tag.Comment);
            FData.Genre := Tag.Genre;
            FData.Track := Ord(Tag.Comment[30]);
          end;
        end;
      end;
    except
      Result := -1;
      ResetData;
    end;
  end;
end;

end.
